
#ifndef SPI_H
#define SPI_H


//------------------------------------------------------------------------------
//         Definitions
//------------------------------------------------------------------------------
//------------------------------ SPI PIO ---------------------------------------
#define SPI0_PORT	AT91C_BASE_PIOA
#define SPI0_NPCS0		12
#define SPI0_NPCS1		13
#define SPI0_NPCS2		14
#define SPI0_NPCS3		15
#define SPI0_MISO		16
#define SPI0_MOSI		17
#define SPI0_SPCK		18
#define SPI0_PIO_MASK	((1<<SPI0_NPCS0)|(1<<SPI0_NPCS1)|(1<<SPI0_NPCS2)|(1<<SPI0_NPCS3)|(1<<SPI0_MISO)|(1<<SPI0_MOSI)|(1<<SPI0_SPCK))

//------------------SPI_Configure - configuration - SPI_MR---------------------
#define  SPI_SLAVE			(0x00 << 0) //  Set SLAVE MODE
#define  SPI_MASTER 		(0x01 << 0) //  Set MASTER SPI

#define  SPI_MULTI_DEVICE	(0x01 << 1) //  SET MULTI DEVICE SUPORT
#define  SPI_ONE_DEVIDE  	(0x00 << 1) //  SET ONE Device Suort

#define  SPI_EN_DECODER		(0x01 << 2) //  Chip Select Decode
#define  SPI_MOD_FDIS       (0x01 << 4) //  Disable Mode Fault Detection
#define  SPI_LLB			(0x10 << 7) //  MISO is Connected to MOSI internally

//---------------------- NPCSx SETTING ---------------------------------------
 // SPI MODE
#define SPI_MODE_0			0x2 // CPOL = 0 & NCPHA = 1
#define SPI_MODE_1			0x0 // CPOL = 0 & NCPHA = 0 
#define SPI_MODE_2			0x3 // CPOL = 1 & NCPHA = 1
#define SPI_MODE_3			0x1 // CPOL = 1 & NCPHA = 0

 // DATA LEN IN SPI
#define SPI_8_BIT			(0x0 << 4)
#define SPI_9_BIT			(0x1 << 4)
#define SPI_10_BIT			(0x2 << 4)
#define SPI_11_BIT			(0x3 << 4)
#define SPI_12_BIT			(0x4 << 4)
#define SPI_13_BIT			(0x5 << 4)
#define SPI_14_BIT			(0x6 << 4)
#define SPI_15_BIT			(0x7 << 4)
#define SPI_16_BIT			(0x8 << 4)
//------------------------------------------------------------------------------
/// "SPI configuration macros"
//------------------------------------------------------------------------------

#define SPI_Configure_PIO_BIT(SPI_PORT,SPI_PIN)		SPI_PORT->PIO_PDR=(1<<SPI_PIN);SPI_PORT->PIO_MDER=(1<<SPI_PIN);SPI_PORT->PIO_ASR=(1<<SPI_PIN);SPI_PORT->PIO_PPUER=(1<<SPI_PIN)
#define SPI_Configure_PIO_MAKS(SPI_PORT,SPI_MASK)	SPI_PORT->PIO_PDR=SPI_MASK;SPI_PORT->PIO_MDER=SPI_MASK;SPI_PORT->PIO_ASR=SPI_MASK;SPI_PORT->PIO_PPUER=SPI_MASK


/// Calculate the PCS field value given the chip select NPCS value
#define SPI_PCS(npcs)       (((~(0x1 << npcs)) & 0xF) << 16)

/// Calculates the value of the SPI_CSR SCBR field given the baudrate and MCK.
#define SPI_SCBR(baudrate, masterClock)		(((unsigned char) (masterClock / baudrate) << 8) & AT91C_SPI_SCBR)

/// Calculates the value of the CSR DLYBS field given the desired delay (in ns)
#define SPI_DLYBS(delay, masterClock)       (((unsigned char) (((masterClock / 1000000) * delay) / 1000) << 16) & AT91C_SPI_DLYBS)

/// Calculates the value of the CSR DLYBCT field given the desired delay (in ns)
#define SPI_DLYBCT(delay, masterClock)      (((unsigned char) (((masterClock / 1000000) * delay) / 32000) << 24) & AT91C_SPI_DLYBCT)
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//         Exported functions
//------------------------------------------------------------------------------

extern void SPI_Enable(AT91S_SPI *spi);
extern void SPI_Disable(AT91S_SPI *spi);
extern void SPI_Reset(AT91S_SPI *spi);

extern void SPI_Configure(AT91S_SPI *spi,
                                 unsigned int id,
                                 unsigned int configuration);
								 
								 
extern void SPI_ConfigureNPCS(AT91S_SPI *spi,
                                     unsigned int npcs,
                                     unsigned int configuration);

									 
extern void SPI_Write(AT91S_SPI *spi, unsigned int npcs, unsigned short data);
extern unsigned char SPI_WriteBuffer(AT91S_SPI *spi,
                                            void *buffer,
                                            unsigned int length);

											
extern unsigned char SPI_IsFinished(AT91S_SPI *pSpi);


extern unsigned short SPI_Read(AT91S_SPI *spi);
extern unsigned char SPI_ReadBuffer(AT91S_SPI *spi,
                                           void *buffer,
                                           unsigned int length);


#endif

